/////////////////////////////////////////////////////////////////////////
// File Name  : calendar.js
// File Type  : JavaScript
// Application: MECA
// Description: This file contains the Month, Day, Item objects and its
//            : corresponding methods. It also contains other supporting 
//            : JavaScript server side functions.
// Created    : 7/29/96
// Copyright  : Copyright 1996 Sapient Corporation
//
/////////////////////////////////////////////////////////////////////////

//
// Name: Month
// Return Value: Month object
// Description: Constructor function for the Month object
//              Used for storing information for a month
//              The Month object consists of the following members
//                 count - number of days in this month
//                 day   - Day list for the Month object
//              The object Month consists of the following methods
//                 add   - method to add item/task of the month
// Output Parameters: none
//
function Month(numOfDays)
{
   this.count = numOfDays;
   this.day = new Day();

   for ( var i = 1 ; i <= numOfDays ; i++ )
   {
      this.day[i] = new Day();
   }
   this.add = AddMonthItem;

   return this;
}

//
// Name: AddMonthItem
// Return Value: none
// Description: Adds an item to a particular day of the month.
// Output Parameters: none
//
function AddMonthItem(dayOfMonth, item)
{
   this.day[dayOfMonth].add(item);
}

//
// Name: Day
// Return Value: Day object
// Description: Constructor function for the Day object 
//              Used for storing information for days of the month
//              The Day object consists of the following members
//                 count - number of items/task for that day
//                 item  - Item list for the Day object
//              The object Day consists of the following methods
//                 add   - method for inserting an item to the day
// Output Parameters: none
//
function Day()
{
   this.count = 0;
   this.item = new Object();
   this.add = AddItem;
}

//
// Name: AddItem
// Return Value: none
// Description: Day object method for inserting an item for a particular
//              day of the month. It also increments the count for number 
//              of item for that day.
// Output Parameters: none
//
function AddItem(item)
{
   this.item[this.count++] = item;
}

// 
// Name: MonthItem
// Return Value: MonthItem object
// Description: Constructor function for the MonthItem object. It stores
//              item/task information for a day of the month.
// Output Parameters: none
//
function MonthItem(loginName, shortDescription, gmtTime)
{
   this.loginName = loginName;
   this.shortDescription = "" + shortDescription + "";
   this.time = gmtTime;
   
   return this;
}

//
// Name: Item
// Return Value: none
// Description: Constructor function for the Item object 
//              Used for storing item/task information for a particular 
//              day of the month.
// Output Parameters: none
//
function Item(eventId, loginName, groupDescription, day, month, year, startDate, endDay, endMonth, endYear, shortDescription, longDescription, time, eventURL)
{
   this.eventId = eventId;
   this.loginName = loginName;
   this.groupDescription = groupDescription;
   this.day = day;
   this.month = month;
   this.year = year;
   this.startDate = startDate;
   this.endDay = endDay;
   this.endMonth = endMonth;
   this.endYear = endYear;
   this.shortDescription = "" + shortDescription + "";
   this.longDescription = "" + longDescription + "";
   this.time = time;
   this.eventURL = eventURL;
   return this;
}

//
// Name: ConnectToDatabase
// Return Value: 0 for error; 1 for successful connection
// Description: Checks if we are still connected to database. If disconnected,
//              then tries to connect again. If unsuccessful, then returns 0
//              indicating error in connecting to the database.
// Output Parameters: none
//
function ConnectToDatabase()
{
   // Check if we are connected to database
   // If not connected to database, then try to connect
   if ( !database.connected() )
   {
      database.connect("INFORMIX", "appalanche_db", "informix", "informix", "meca")
      // If we are still unable to connect to database, then return 0
      // indicating no connection
      if ( !database.connected() )
         return 0;
   }
 
   // We are connected to database, so return 1 indicating connection
   return 1;
}

//
// Name: GetDayInfo
// Return Value: Day object
// Description: Function to get items/task for a particular day of the month
//              It gets information depending on the groups the user has
//              subscribed to.
// Output Parameters: none
//
function GetDayInfo(selectedYear, selectedMonth, selectedDay)
{
   // Connect to the database
   if ( !ConnectToDatabase() )
   {
      ShowError("Unable to connect to database")
      return 0;
   }
   else 
   {
      // Create an instance of the Day object
      var currDay = new Day();

      // Prepare query to find out the users subscribed list of groups
      var aclQuery = "SELECT groupName FROM accesscontrollist ";
          aclQuery += "WHERE loginName = '" + client.loginName + "' ";
          aclQuery += "AND subscribedFlag = 'Y'";
 
      aclCursor = database.cursor(aclQuery);

      // Define the query to get day info
      var dayQuery = "SELECT groupDescription, eventId, Events.loginName loginName, DAY(eventStartDate) day, MONTH(eventStartDate) month, YEAR(eventStartDate) year, ";
          dayQuery += "eventStartDate, DAY(eventStopDate) stopDay, MONTH(eventStopDate) stopMonth, YEAR(eventStopDate) stopYear, ";
          dayQuery += "shortDescription, longDescription, eventGMTTime, eventURL ";
          dayQuery += "FROM Events, MecaGroup WHERE  eventStartDate <= DATE('" + selectedMonth + "/" + selectedDay + "/" + selectedYear + "') ";
          dayQuery += "AND eventStopDate >= DATE('" + selectedMonth + "/" + selectedDay + "/" + selectedYear + "') ";
          dayQuery += "AND Events.loginName = MecaGroup.loginName ";
   
      // Initialize the count of subscribed groups for the user
      var aclCount = 0;

      // Go into a loop to find the users subscribed groups
      while ( aclCursor.next() )
      {
         if ( aclCount == 0 )
         {
            dayQuery += "AND ((Events.loginName = '" + aclCursor.groupName + "') ";
         }
         else
         {
            dayQuery += "OR (Events.loginName = '" + aclCursor.groupName + "') ";
         }
         aclCount++;
      }

      // Close cursor
      aclCursor.close();

      // If the user has not subscribed to a group (aclCount == 0),
      // then do not retrieve any information from the database.
      // else retrieve all events that are associated with the groups
      // the user has subscribed to.
      if ( aclCount > 0 )
      {
         dayQuery += ") ";

         dayQuery += " ORDER BY eventGMTTime";

         // Define and open a cursor to get items/task for each day of the month
         var dayCursor = database.cursor(dayQuery);

         // Loop till you access all the events for the day for the subscribed
         // groups for the user.
         while ( dayCursor.next() )
         {
             dayItem = new Item(dayCursor.eventId, dayCursor.loginName, dayCursor.groupDescription, dayCursor.day, dayCursor.month, dayCursor.year, dayCursor.eventStartDate, 
dayCursor.stopDay, dayCursor.stopMonth, dayCursor.stopYear, dayCursor.shortDescription, dayCursor.longDescription, dayCursor.eventGMTTime, dayCursor.eventURL);
             currDay.add(dayItem);
         }
      
         // Close the cursor
         dayCursor.close();
      }   

      // Return the day info
      return currDay;
   }
}

//
// Name: PrintDayInfo
// Return Value: none
// Description: Debug function which prints the information stored in the
//              Day object
// Output Parameters: none
//
function PrintDayInfo(currDay)
{
   write("# of items = " + currDay.count + "<BR>\n");
   itemCount = currDay.count;
   if ( itemCount > 0 )
   {
      write("<BR># of items for the day = " + itemCount);
      write("<UL>\n");
      for ( var l = 0 ; l < itemCount ; l++ )
      {
         write("<LI> short = " + currDay.item[l].shortDescription + "</LI>\n");
         write("<LI> login Name  = " + currDay.item[l].loginName + "</LI>\n");
      }
      write("</UL>\n");
   }
}

//
// Name: PrintMonthInfo
// Return Value: none
// Description: Debug function which prints the information stored in the
//              Month object
// Output Parameters: none
//
function PrintMonthInfo(currMonth)
{
   write("# of days = " + currMonth.count + "<BR>\n");

   for ( var k = 1 ; k <= currMonth.count ; k++ )
   {
      write("Day = " + k + "<BR>\n");
      PrintDayInfo(currMonth.day[k]);
   }
}
   
//
// Name: GetMonthInfo
// Return Value: Month object
// Description: This function gets the information for a particular month of
//              a year. It creates the Month object and stores the information
//              for that month in the created object. It just gets the bare
//              minimum information needed for displaying the calendar.
// Output Parameters: none
//
function GetMonthInfo(selectedMonth, selectedYear)
{
   // Connect to the database
   if ( !ConnectToDatabase() )
   {
      ShowError("Unable to connect to database")
      return 0;
   }
   else 
   {
      // Get the number of days in the month
      var numOfDays = MyMonthDays(selectedMonth, selectedYear);
 
      // Temp varialbe, as we use it pretty often
      // We add 1 as month value passed to the function is from 0-11
      var convertedMonth = eval(selectedMonth+1);
 
      // Temp variables for database query
      var startOfMonth = "'" + convertedMonth + "/" + "01" + "/" + selectedYear + "'";
      var endOfMonth = "'" + convertedMonth + "/" + numOfDays + "/" + selectedYear + "'";

      // Get epoch time of the start and end of the month
      // This is used later on for time comparison
      var startOfMonthDate = new Date(selectedYear, selectedMonth, 1);
      var endOfMonthDate = new Date(selectedYear, selectedMonth, numOfDays);
      var startOfMonthTime = startOfMonthDate.getTime();
      var endOfMonthTime = endOfMonthDate.getTime();
 
      // Create an instance of the Month object
      var currMonth = new Month(numOfDays);

      // Prepare query to find out the users subscribed list of groups
      var aclQuery = "SELECT groupName FROM accesscontrollist ";
          aclQuery += "WHERE loginName = '" + client.loginName + "' ";
          aclQuery += "AND subscribedFlag = 'Y'";

      aclCursor = database.cursor(aclQuery);

      // Define the query to get month info
      // Get every item for the month
      var monthQuery = "SELECT DISTINCT DAY(eventStartDate) startDay, MONTH(eventStartDate) startMonth, YEAR(eventStartDate) startYear, " 
          monthQuery += "DAY(eventStopDate) stopDay, MONTH(eventStopDate) stopMonth, YEAR(eventStopDate) stopYear, "
          monthQuery += "loginName, shortDescription, eventGMTTime ";
          monthQuery += "FROM events "
          monthQuery += "WHERE  (((MONTH(eventStartDate) = '" + convertedMonth + "' ";
          monthQuery += "AND YEAR(eventStartDate) = '" + selectedYear + "' ) ";
          monthQuery += "OR (MONTH(eventStopDate) = '" + convertedMonth + "' ";
          monthQuery += "AND YEAR(eventStopDate) = '" + selectedYear + "' ) ) ";
          monthQuery += "OR (eventStartDate < DATE(" + startOfMonth + ") ";
          monthQuery += "AND eventStopDate > DATE(" + endOfMonth + "))) ";

      // Initialize the count of subscribed groups for the user
      var aclCount = 0;

      // Go into a loop to find the users subscribed groups
      while ( aclCursor.next() )
      {
         // If it's the first one, then you need an AND condition
         if ( aclCount == 0 )
         {
            monthQuery += "AND ((loginName = '" + aclCursor.groupName + "') ";
         }
         // else you need an OR condition
         else
         {
            monthQuery += "OR (loginName = '" + aclCursor.groupName + "') ";
         }
         aclCount++;
      }

      // Close cursor
      aclCursor.close();

      // If the user has not subscribed to a group (aclCount == 0),
      // then do not retrieve any information from the database.
      // else retrieve all events that are associated with the groups
      // the user has subscribed to.
      if ( aclCount > 0 )
      {
         monthQuery += ") ";
       
         monthQuery += "ORDER BY eventGMTTime";

         // Define and open a cursor to get items for each day of the month
         var monthCursor = database.cursor(monthQuery);
      
         // Loop till you access all the events for the month for the subscribed
         // groups for the user.
         while ( monthCursor.next() )
         {
            // Get epoch time for the start and stop date of the items
            // This is later used for comparison
            var startDate = new Date(monthCursor.startYear, eval(monthCursor.startMonth-1), monthCursor.startDay);
            var stopDate = new Date(monthCursor.stopYear, eval(monthCursor.stopMonth-1), monthCursor.stopDay);
            var startDateTime = startDate.getTime();
            var stopDateTime = stopDate.getTime();

            monthItem = new MonthItem(monthCursor.loginName, monthCursor.shortDescription, monthCursor.eventGMTTime);

            // We need to check the duration of the items.
            // The duration can be either one day, or multiple days or could
            // crossing boundaries i.e. could be starting and/or ending in
            // different months. 
            
            // Check if it's multi-day items or single day items
            // If it's single day items, then we just enter in the
            // Month object.
            if ( startDateTime != stopDateTime )
            {
               // It's a multi-day item.
               // Check if it's within the same month.
               if ( monthCursor.startMonth == monthCursor.stopMonth && monthCursor.startYear == monthCursor.stopYear )
               {
                  // If it's within the same month, then calculate the different,
                  // and add the item as many times from startDay.
                  var diffInDays = monthCursor.stopDay - monthCursor.startDay + 1;
                  for ( var i = 0 ; i < diffInDays ; i++ )
                     currMonth.add(eval(monthCursor.startDay+i), monthItem);
               }
               // It's not within the same month.
               // i.e. a> either starts in a previuos month/year and ends in this month
               //      b> or starts in this month, and ends in next month/year
               //      c> or starts in a previuos month/year and ends in the next month/year
               else
               {
                  // Check if the start time is before the start of the month, 
                  // and the stop time is before the month ends.
                  if ( startDateTime < startOfMonthTime && stopDateTime < endOfMonthTime )
                  { 
                     // Add item to days from 1st of month to the end day.
                     var dayToStartOfMonth = monthCursor.stopDay; 
                     for ( var j = 1 ; j <= dayToStartOfMonth ; j++ )
                        currMonth.add(j, monthItem);
                  }
                  // Check if the stop time is after the end of the month, 
                  // but the start time is after the start of the month.
                  else if ( stopDateTime > endOfMonthTime && startDateTime > startOfMonthTime )
                  { 
                     // Add item to days from start day to the end of month.
                     var dayToEndOfMonth = monthCursor.startDay; 
                     for ( var k = dayToEndOfMonth ; k <= numOfDays ; k++ )
                        currMonth.add(k, monthItem);
                  }
                  // Check if the start time is before the start of the month,
                  // and the stop time is after the end of the month
                  else if ( startDateTime < startOfMonthTime && stopDateTime > endOfMonthTime )
                  {
                     // Add item to all the days of the month
                     for ( var l = 1 ; l <= numOfDays ; l++ )
                        currMonth.add(l, monthItem);
                  }
               }
            }
            // else it is a single day item
            else
            {
               // Add it to the day.
               currMonth.add(monthCursor.startDay, monthItem);
            }
         }

         // Close cursor
         monthCursor.close();
      }

      // Return the stored info
      return currMonth;
   }
}

//
// Name: Months
// Return Value: Month array
// Description: This function creates a month Array.
//              The months start from 0 (January) to 11 (December)
// Output Parameters: none
//
function Months()
{
   this[0] = "January"
   this[1] = "February"
   this[2] = "March"
   this[3] = "April"
   this[4] = "May"
   this[5] = "June"
   this[6] = "July"
   this[7] = "August"
   this[8] = "September"
   this[9] = "October"
   this[10] = "November"
   this[11] = "December"

   return this;
}

//
// Name: DrawCalendar
// Return Value: Month array
// Description: Draw the calendar on the HTML form based on the 
//              monthNum (0-11) and the four digit year.
// Output Parameters: none
//
function DrawCalendar(currMonth, monthNum, year)
{
    // Set client variables to be used by the day view screen.
    // Signifies what month and year the client is looking at.
    client.monthChoice = monthNum;
    client.yearChoice = year;

    var tempDate = new Date (year, monthNum, 1);	

    var firstDay = tempDate.getDay();

   // Write blank HTML cells until we get to the first of the month
   for (var tableCounter = 0; firstDay > tableCounter; tableCounter++ )
   {
       write("<TD HEIGHT=57 WIDTH=14%></TD>");
   }
	
   var count = firstDay + 1;

   // Go thru each day and create the day cells in the table
   for ( var dayCount = 1 ; dayCount <= currMonth.count ; dayCount++ )
   {
      itemCount = currMonth.day[dayCount].count;
      if (dayCount == client.currentDate && client.currentMonth == monthNum)
      {
        write("<TD BGCOLOR=#C4FFD2 HEIGHT=57 WIDTH=14%>")
      }
      else 
      {
        write("<TD HEIGHT=57 WIDTH=14%>")
      }
      write ("<B><A HREF=\"setday.htm?dateChoice=" + dayCount + "\">" + dayCount + "</B></A><BR>");

      // Check if there are any items for that day
      if ( itemCount > 0 )
      {
         // Check if there are more than 2 items for that day
         // If there are more than 2 items, then print 2 items
         // and also put "more..." to indicate that there are more items
         // for this day
         // If there are 3 or less items, then print those.
         var maxItems = 3;
         if ( itemCount > maxItems)
         {
            for ( var k = 0 ; k < maxItems ; k++ )
            {
               write("<IMG SRC=\"images/bullet.gif\" height=8 width=10><FONT SIZE= -2><FONT COLOR=#000000>" + currMonth.day[dayCount].item[k].time + " " + currMonth.day[dayCount].item[k].shortDescription + "</FONT><BR>");
            }
            write("<FONT size=-1> more...</FONT>");
         }
         else
         {
            for ( var l = 0 ; l < itemCount ; l++ )
            {
               write("<IMG SRC=\"images/bullet.gif\" height=8 width=10><FONT size= -2><FONT COLOR=#000000>" + currMonth.day[dayCount].item[l].time + " " + currMonth.day[dayCount].item[l].shortDescription + "</FONT><BR>");   
   
            }
         }
      }
      write("</TD>\n");
      // You want to start on a new row/line after every seven days
      if (count == 7)
      {
         write("</TR><TR VALIGN = top>");
         count = 0;
      }
      count = count + 1;
   }
}	

//
// Name: MyMonthDays
// Return Value: Number of days in the month
// Description: Calculates the number of days in the month 
// Output Parameters: none
//
function MyMonthDays(monthNum, year)
{ 
   // There will always be at least 27 days in the month
   var tempDate = new Date(year, monthNum, 27);
   
   for (var currentDate = 27; tempDate.getDate() == currentDate; currentDate++)
   {
      tempDate.setDate(currentDate + 1);
   }
   return currentDate - 1;
}

//
// Name: AllPermissions
// Return Value: (1 if user has permission to modify the event, 0 otherwise)
// Description:  This is called to check if the user has permissions on an
//               event or not.  The eventId is passed to identify the event.
//
function AllPermissions(eventId)
{
   // Check if the user has delete permissions to the group
   // Check if the group has 'A' (ALL) permissions
   // Or if the user is the owner of the group, in which case the user
   // has ALL permissions.
   var permissionQuery = "SELECT mg.groupPermission FROM MecaGroup mg, Events e "
       permissionQuery += "WHERE e.eventId = " + eventId + " ";
       permissionQuery += "AND e.loginName = mg.loginName ";
       permissionQuery += "AND (mg.groupPermission = 'A' ";
       permissionQuery += "OR mg.groupOwner = '" + client.loginName + "')";

   // Execute the query
   var permissionCursor = database.cursor(permissionQuery);

   // If the user has permissions to delete the events, then delete it.
   if ( permissionCursor.next() )
   {  
      permissionCursor.close();
      return 1;
   }

   permissionCursor.close();
   return 0
}

//
// Name: DeleteTask
// Return Value: none
// Description: This function deletes a task from the database.  The 
//              eventId of the task to be deleted is passed as an argument.
// Output Parameters: none
//
function DeleteTask(eventId)
{
   // Connect to the database, if we are not connected
   if ( !ConnectToDatabase() )
   {
      ShowError("Could not connect to database");
   }
   else
   {
      // Check if it has delete permissions on the group
      if ( AllPermissions(eventId) )
      {
         // Prepare the query
         var deleteQuery = "DELETE FROM events WHERE eventId = " + eventId;
   
         // Execute the query
         var dbStatus = database.execute(deleteQuery);
      }
      else
         ShowError("You do not have delete permissions.");
   }
}

//
// Name: AddSubscribableGroups
// Return Value: none
// Description:  This is called after the select tag to list the groups that
//               the user has access to, but also to show the list of 
//               subscribed groups as selected.
//              
function AddSubscribableGroups() 
{
   // Connect to the database, if we are not connected
   if ( !ConnectToDatabase() )
   {
      ShowError("Could not connect to database");
   }
   else 
   {
      //  Find the list of groups that are either global OR where the access
      //  receiver (ACL.loginName) is the current user (client.loginName)
      //  AND the groupType is private AND the group's name (MG.loginName)
      //  is the same as the group giving access. (ACL.groupName)
      var groupQuery  = "SELECT DISTINCT MecaGroup.groupDescription, MecaGroup.loginName"
          groupQuery += " FROM MecaGroup, AccessControlList"
          groupQuery += " WHERE (MecaGroup.groupType='G')"
          groupQuery += " OR (AccessControlList.loginName='" + client.loginName + "'"
          groupQuery += " AND MecaGroup.groupType='L'"
          groupQuery += " AND MecaGroup.loginName=AccessControlList.groupName)"
          groupQuery += " ORDER BY MecaGroup.groupDescription"

      var groupCursor = database.cursor(groupQuery);

      while ( groupCursor.next() )
      {
         //  For each one that the user has access to, find out if the
         //  user has already subscribed to it or not.  This is necessary as
         //  a second select since ACL doesn't include global groups.
         var subscribedQuery  = "SELECT subscribedFlag FROM AccessControlList"
             subscribedQuery += " WHERE groupName='" + groupCursor.loginName + "'"
             subscribedQuery += " AND loginName='" + client.loginName + "'"

         var subscribedCursor = database.cursor(subscribedQuery);

         write("<OPTION ");

         //  If it is subscribed to, write selected for the item
         if ((subscribedCursor.next()) && (subscribedCursor.subscribedFlag == 'Y'))  
            write("SELECTED ");

         write("VALUE=" + groupCursor.loginName + ">");
         write(groupCursor.groupDescription + " ");

         subscribedCursor.close()
      }

      groupCursor.close();
    }
}


//
// Name: SubscribeGroups
// Return Value: none
// Description:  This takes a string containing the list of groups (delimited by *'s)
//               and subscribes the current user to those groups.  It does so by 
//               removing all subscriptions and then parsing the string.
//
//  Note: SubmitGroups is the complementary function which creates the groups string
//        A user can subscribe to groups which:
//  1. are global
//  2. the user owns
//  3. another user has given the user access to
//               
// Output Parameters: none
//
function SubscribeGroups(groups) 
{
   // Connect to the database, if we are not connected
   if ( !ConnectToDatabase() )
   {
      ShowError("Could not connect to database");
   }
   else 
   {
      //  Get groupType and group's name from Meca Group where the current user has
      //  access.  This will include global groups if the user has subscribed to it
      //  in the ACL table.

      var groupQuery  = "SELECT DISTINCT MecaGroup.groupType, MecaGroup.loginName"
          groupQuery += " FROM MecaGroup, AccessControlList"
          groupQuery += " WHERE AccessControlList.loginName='" + client.loginName + "'"
          groupQuery += " AND MecaGroup.loginName=AccessControlList.groupName"

      var groupCursor = database.cursor(groupQuery);

      //  Loop through the list of all accessible groups for the user.  And re-initialize
      //  ACL where global groups are removed and all subscribed flags are turned off. (N)
      while ( groupCursor.next() )
      {

        if (groupCursor.groupType == 'G')
        {
            //  Delete global groups in loginName's ACL

            deleteQuery  = "DELETE FROM AccessControlList"
            deleteQuery += " WHERE groupName = '" + groupCursor.loginName + "'"
            deleteQuery += " AND loginName = '" + client.loginName + "'"
            database.execute(deleteQuery)
        }
        else
        {
            //  Update non-global groups in the ACL with subscribed flag = no

            updateQuery  = "UPDATE AccessControlList"
            updateQuery += " SET subscribedFlag='N'"
            updateQuery += " WHERE groupName = '" + groupCursor.loginName + "'"
            updateQuery += " AND loginName = '" + client.loginName + "'"
            database.execute(updateQuery)
        }
      }
      
      //  Now parse the string and subscribe to groups that the user wants to subscribe to.
      for (i = groups.indexOf("*"); i; groups = groups.substring(i + 1, groups.length), i = groups.indexOf("*")) 
      {
         var group = groups.substring(0, i);
         if (group.length >= 1)
         {
            //  Check if group already exists in the ACL
            var checkQuery  = "SELECT * FROM AccessControlList WHERE groupName = '" + group + "'"
                checkQuery += " AND loginName = '" + client.loginName + "'"

            var checkCursor = database.cursor(checkQuery)

            //  if already in there then update the flag.
            if (checkCursor.next())
            {
               updateQuery  = "UPDATE AccessControlList"
               updateQuery += " SET subscribedFlag = 'Y'"
               updateQuery += " WHERE groupName = '" + group + "'"
               updateQuery += " AND loginName = '" + client.loginName + "'"
               database.execute(updateQuery)
            }
            else
            {
               //  if it is not in the ACL, then add it (because it must be global).
               var insertACLQuery  = "INSERT INTO AccessControlList (loginName, subscribedFlag, groupName) " 
                   insertACLQuery += " VALUES('" + client.loginName + "','Y','"+ group +"')";

               database.execute(insertACLQuery);
            }
         }
      }
   }
   redirect("month.htm");
}


//
// Name: AddModifiableGroups
// Return Value: none
// Description:  This generates an option list for a select form element.  The list includes 
//               all groups that are write accessible to the user that is currently logged 
//               into MECA.  Issues to resolve:
//   1) The current user is hard-coded
//   2) error-handling
//               
// Output Parameters: none
//
function AddModifiableGroups(loginName) 
{
    // Connect to the database, if we are not connected
    if ( !ConnectToDatabase() )
    {
        ShowError("Unable to connect to database");
    }
    else 
    {
        Query  = "SELECT DISTINCT MecaGroup.loginName mgLoginName, MecaGroup.groupDescription mgGroupDescription"
        Query += " FROM MecaGroup, AccessControlList"
        Query += " WHERE"
        Query += " (MecaGroup.groupType='G' AND MecaGroup.groupPermission='A')"
        Query += " OR"
        Query += " (MecaGroup.groupOwner='" + client.loginName + "')"
        Query += " OR"
        Query += " (AccessControlList.loginName='" + client.loginName + "' AND MecaGroup.groupPermission='A'"
        Query += " AND AccessControlList.groupName = MecaGroup.loginName)"
        Query += " ORDER BY MecaGroup.groupDescription"

        Cursor = database.cursor(Query);

        // Loop through cursor
        while (Cursor.next())
        {
            if ( loginName == Cursor.mgLoginName )
               write("<OPTION VALUE=" + Cursor.mgLoginName + " SELECTED>" + Cursor.mgGroupDescription);
            else
               write("<OPTION VALUE='" + Cursor.mgLoginName + "'>" + Cursor.mgGroupDescription);
        }

        Cursor.close();
    }
}

//
// Name: MecaGroup
// Return Value: returns an object with values passed in as parameters.
// Description:  Allows values to be passed in and grouped as output to the function.
// Output Parameters: none
//
function MecaGroup(loginName, loginPassword, groupOwner, groupDescription, groupType, groupPermission)
{
   this.loginName = loginName;
   this.loginPassword = loginPassword;
   this.groupOwner = groupOwner;
   this.groupDescription = groupDescription;
   this.groupType = groupType;
   this.groupPermission = groupPermission
}

//
// Name: GetGroupInfo
// Return Value: object containing MecaGroup data.
// Description:  Use MecaGroup above to create objects with the MecaGroup information.
// Output Parameters: none
//
function GetGroupInfo(loginName)
{
   if ( !ConnectToDatabase() )
   {
      ShowError("Could not connect to database");
   }
   else
   {
      var groupInfoQuery = "SELECT * FROM MecaGroup where loginName = '" + loginName + "'";
      var groupInfoCursor = database.cursor(groupInfoQuery);

      while ( groupInfoCursor.next() )
      {
         var groupInfo = new MecaGroup(groupInfoCursor.loginName, groupInfoCursor.loginPassword, groupInfoCursor.groupOwner, groupInfoCursor.groupDescription, groupInfoCursor.groupType, groupInfoCursor.groupPermission);
      }
  
      groupInfoCursor.close();

      return groupInfo;
   }
}

//
// Name: InsertAcess
// Return Value: none
// Description:  Insert the groups user has given access to in the access
//               control list.
// Output Parameters: none
//
function InsertAccess(loginName, groupName, subscribedFlag)
{
   //  Perform the simple insert.
   var insertQuery  = "INSERT INTO AccessControlList ";
       insertQuery += " (loginName, groupName, subscribedFlag) "
       insertQuery += " VALUES "
       insertQuery += " ('" + loginName + "', '" + groupName + "', '" + subscribedFlag + "')"
   
   dbStatus = database.execute(insertQuery);

   if ( dbStatus )
   {
      ShowError("Could not insert group access");
   }
}

//
// Name: InsertUpdateGroup
// Return Value: none
// Description:  This will insert or update group information in the database.
//               actionType == 'Default' means that the user pushed the add group
//               button on the initial login screen.
// Output Parameters: none
//
function InsertUpdateGroup(loginName, loginPassword, groupOwner, groupDescription, groupType, groupPerm, actionType, groups)
{
   if ( !ConnectToDatabase() )
   {
      ShowError("Could not connect to database");
   }
   else
   {
      //  If the groupOwner returned SELF from the group info form, then the 
      //  groupOwner is the logged person.
      if (groupOwner == 'SELF')
      {
         groupOwner = loginName
      }
   
      //  If this is a new group, do an insert.  If this is a modify, do an update.
      if (actionType == 'New' || actionType == 'Default')
      {
         //  For the special case of New (meaning that the new group request did *not*
         //  come from the initial login screen, then check for prior group name.
         if (CheckGroupExistence(loginName) == 0)
         {
            write("<SCRIPT>\n");
            write("   alert('ERROR: Group " + loginName + " already exists. Please select another group name.');\n");
            write("   history.back()\n");
            write("</SCRIPT>\n");
            return;
         }
      
         var insertQuery  = "INSERT INTO MecaGroup ";
             insertQuery += " (groupType, loginName, loginPassword, groupOwner, groupDescription, groupPermission) ";
             insertQuery += " VALUES "
             insertQuery += " ('" + groupType + "', '" + loginName + "', '" + loginPassword + "', ";
             insertQuery += " '" + groupOwner + "', '" + groupDescription + "', '" + groupPerm + "')";
    
         dbStatus = database.execute(insertQuery);
      }
      else 
      {
         var updateQuery  = "UPDATE MecaGroup ";
             updateQuery += " SET loginPassword = '" + loginPassword + "',";
             updateQuery += " groupOwner = '" + groupOwner + "',";
             updateQuery += " groupDescription = '" + groupDescription + "',";
             updateQuery += " groupType = '" + groupType + "', ";
             updateQuery += " groupPermission = '" + groupPerm + "' ";
             updateQuery += " WHERE loginName = '" + loginName + "'";

         dbStatus = database.execute(updateQuery);
      
         // First, delete all entries in the ACL where the groupName is the 
         //   entered groupName.  This is done to clear out the ACL for modifies
         //   since a modify means that there is old data in the ACL table.

         var delPerm  = "DELETE FROM AccessControlList "
             delPerm += " WHERE groupName = '" + loginName + "'"
             delPerm += " OR loginName = '" + loginName + "'"

         dbStatus = database.execute(delPerm)
      }

      // If the type is global, no access stuff needs to be done.
      if (groupType == 'G')
      {
         if (actionType != 'Default')
           {
             redirect("month.htm"); 
           }
         else
           {
             redirect("init.htm")
           }
         return;  // Just to make the point of leaving this function.
      }


      //  Okay.  It is a private type.  We need to add this or modify this
      //  group's information in the MecaGroup and ACL tables.


      var addPerm  = "INSERT INTO AccessControlList "
          addPerm += " (loginName, groupName, subscribedFlag) "
          addPerm += " VALUES "

      //  If the name of the group is different than the owner, add a line
      //  for the owner with access to the name of the group.  This way,
      //  the owner can have potential access to the name of the group.
      //  Example:  groupA owned by groupB will have an entry for 
      //            groupA with access to groupB and
      //            groupA with access to groupA so that groupA can
      //            have potential access to itself.
      if (loginName != groupOwner)
      {
         var addGroupPerm = addPerm
             addGroupPerm += " ('" + groupOwner  + "', '" + loginName + "', 'N')" 

         dbStatus = database.execute(addGroupPerm);
      }

      //  Here, we are letting groupA owned by groupA have potential access.
      addPerm += " ('" + loginName + "', '" + loginName + "', 'N')" 

      dbStatus = database.execute(addPerm);


      //  Parse out the list of groups with potential access.
      for (i = groups.indexOf("*"); i; groups = groups.substring(i + 1, groups.length), i = groups.indexOf("*"))
      {
        group = groups.substring(0, i);
        if (group != loginName && group != groupOwner)
        {
            InsertAccess(group, loginName, 'N')
        }
      }

      //  If this came from the create group button on the initial screen,
      //  then go back to that screen.
      if (actionType == 'Default')
      {
         redirect("init.htm");
      }
      else
      {
         redirect("month.htm");
      }
   }
}


//
// Name: OwnerList
// Return Value: returns the object OwnerList.
// Description:  Create a data structure for owner information.
// Output Parameters: none
//
function OwnerList()
{
   this.count = 0;
   this.ownerRec = new Object();

   //  This is intended to be a pointer to the function.  When add is
   //  called elsewhere, it should include the input parameter for AddOwnerRec.
   this.add = AddOwnerRec;

   return this;
}

//
// Name: AddOwnerRec
// Return Value: returns the next owner record.
// Output Parameters: none
//
function AddOwnerRec(ownerRec)
{
   this.ownerRec[this.count++] = ownerRec;
}
 
//
// Name: Owner
// Return Value: returns the Owner object.
// Description: Constructor function for the Owner object.
// Output Parameters: none
//
function Owner(loginName, groupDescription)
{
   this.loginName = loginName;
   this.groupDescription = groupDescription;
   
   return this;
}

//
// Name: GetOwnerList
// Return Value: returns the list of owners.
// Description:  Select where the owner is the given owner.  Then create the 
//               record list of groups owned.
// Output Parameters: none
//
function GetOwnerList(selectedOwner)
{
   if ( !ConnectToDatabase() )
   {
      ShowError("Could not connect to database");
   }
   else
   {
      var ownerList = new OwnerList();

      var ownerQuery  = "SELECT loginName, groupDescription FROM MecaGroup ";
          ownerQuery += " WHERE groupOwner = '" + selectedOwner + "'";

      var ownerCursor = database.cursor(ownerQuery);

      while ( ownerCursor.next() )
      {
          ownerRec = new Owner(ownerCursor.loginName, ownerCursor.groupDescription);
          ownerList.add(ownerRec);
      }
      
      // Close the cursor
      ownerCursor.close();

      // Return the owner list
      return ownerList;
   }
}

//
// Name: CreateGroupList
// Return Value: none
// Description:  Create the selection box using the list of owners information.
// Output Parameters: none
//
function CreateGroupList(ownerList)
{
   var ownerCount = ownerList.count;

   write("<SELECT NAME=ownerGroupList>\n");

   if ( ownerCount > 0 )
   {
      for ( var i = 0 ; i < ownerCount ; i++ )
      {
         if ( i == 0 )
         {
            write("  <OPTION VALUE=" + ownerList.ownerRec[i].loginName + " SELECTED>" + ownerList.ownerRec[i].groupDescription + "\n");
         }
         else
         {
            write("  <OPTION VALUE=" + ownerList.ownerRec[i].loginName + ">" + ownerList.ownerRec[i].groupDescription + "\n");
         }
      }
   }
   write("</SELECT>\n");
}
 
//
// Name: DeleteGroup
// Return Value: none
// Description:  Called from grpdelete.htm.  Delete a group, all the 
//               groups that group owns (if any), all the accesses 
//               in the AccessControlList table, and all events 
//               associated with that group and its children.
//               Show no mercy.
// Output Parameters: none
//
function DeleteGroup(loginName)
{
   if ( !ConnectToDatabase() )
   {
      ShowError("Could not connect to database");
   }
   else
   {
      // Query to determine wheter or not selected group is a parent group  

      var Query  = "SELECT loginName FROM MecaGroup "
          Query += " WHERE loginName = '" + loginName + "'"
          Query += " AND groupOwner = loginName "

      qResult = database.cursor(Query)

      //
      //  if qResult.next exists then this is a parent group.  Need to delete
      //  everything associated with this parent and everything associated with
      //  this parent's children.
      //
      if (qResult.next())
      {
         // tempLoginName is the loginName of the parent group. Need to deal
         // with the children first.

         var tempLoginName = qResult.loginName

         //  Find all the children.
         var selectQuery = "SELECT loginName FROM MecaGroup WHERE groupOwner = '" + tempLoginName + "'"

         children = database.cursor(selectQuery)

         while (children.next())
         {
            deleteGroupQuery = "DELETE FROM MecaGroup WHERE loginName = '" + children.loginName + "'"
            deleteEventQuery = "DELETE FROM Events WHERE loginName = '" + children.loginName + "'"
            deleteACLQuery = "DELETE FROM AccessControlList WHERE loginName = '" + children.loginName + "'"
            deleteACLQuery += " OR groupName = '" + children.loginName + "'"
            dbStatus = database.execute(deleteACLQuery)
            dbStatus = database.execute(deleteEventQuery)
         }  dbStatus = database.execute(deleteGroupQuery)

         children.close()

         //  Delete the parent stuff.

         deleteACLQuery = "DELETE FROM AccessControlList WHERE loginName = '" + tempLoginName + "'"
         deleteACLQuery += " OR groupName = '" + tempLoginName + "'"
         deleteEventQuery = "DELETE FROM Events WHERE loginName = '" + tempLoginName + "'"
         deleteGroupQuery = "DELETE FROM MecaGroup WHERE groupOwner = '" + tempLoginName + "'"
         dbStatus = database.execute(deleteACLQuery)
         dbStatus = database.execute(deleteEventQuery)
         dbStatus = database.execute(deleteGroupQuery)
   
         redirect("init.htm")
      }
      else
      {
         //  Just delete this child group.

         var deleteGroupQuery = "DELETE FROM MecaGroup WHERE loginName = '" + loginName + "'"
         var deleteEventQuery = "DELETE FROM Events WHERE loginName = '" + loginName + "'"
         var deleteACLQuery = "DELETE FROM AccessControlList WHERE loginName = '" + loginName + "'"
         deleteACLQuery += " OR groupName = '" + loginName + "'"
         dbStatus = database.execute(deleteACLQuery)
         dbStatus = database.execute(deleteEventQuery)
         dbStatus = database.execute(deleteGroupQuery)
         
         redirect("grpadmin.htm")
      }
   }
}

//
// Name: CheckGroupExistence
// Return Value: none
// Description:  Check if group name already exists before we add that 
//               group to the database.
// Output Parameters: none
//
function CheckGroupExistence(groupName)
{

   if ( !ConnectToDatabase() )
   {
      ShowError("Could not connect to database");
   }
   else
   {
      var selectQuery  = "SELECT loginName FROM MecaGroup "
          selectQuery += " WHERE loginName = '" + groupName + "'"

      groupCursor = database.cursor(selectQuery);

      if ( groupCursor.next() )
      {
         groupCursor.close();
         return 0;
      }
      groupCursor.close();
   }
   return 1;
}

//
// Name: GetData
// Return Value: none
// Description:  Show all groups that can have potential access to the inputted group.
// Output Parameters: none
//
function GetData(isNew, loginName)
{
   if (!ConnectToDatabase())
   {
      ShowError("Unable to connect to the database")
   }
   else
   {
      //  Select all groups except for yourself on a modify.

      Query  = "SELECT loginName, groupDescription FROM MecaGroup"
      if ( isNew != 'New' && isNew != 'Default' )
      {
         Query += " WHERE loginName != '" + loginName + "'"
      }
      Query += " ORDER BY groupDescription"
      Cursor = database.cursor(Query)

      //
      //  Create a function within script tags for use by a select tag
      //  that shows all the loginName's from the ACL for a given group.
      //

      //  Loop through the list of groups above and show all groups (ACL.loginName)
      //  that have access to the inputted group (ACL.groupName).
      while(Cursor.next())
      {
      
         Query = "SELECT loginName FROM AccessControlList WHERE groupName = '" + loginName + "'";
         Query += " AND loginName = '" + Cursor.loginName + "'"
         accessCursor = database.cursor(Query);

         write("<OPTION");
         if (accessCursor.next())
            write(" SELECTED");
         write(" VALUE=" + Cursor.loginName + ">");
         write(Cursor.groupDescription + "\n");
         accessCursor.close();
      }
      Cursor.close()
   }
}

//
// Name: ShowError
// Return Value: none
// Description: Takes an argument and draws a screen showing error message.
// Output Parameters: none
//
function ShowError(message)
{
   write("<HTML><BODY BGCOLOR=#FFFFFF>")
   write("<H3>")
   write("   Error: " + message + "<BR>")
   write("</H3>")
   write("<HR>")
   write("<CENTER>")
   write("   Press 'Back' to go back to the previous page")
   write("</CENTER>")
   write("</BODY></HTML>")
   exit
}

// Name:  TimeOut
// Return Value: redirects user to login screen(init.htm) if they've timed out.
// Output Parameters: none
function TimeOut()
{
   if (client.loginName == null || client.loginName == '')
        redirect("init.htm")
}
